use std::collections::HashMap;
use serde::{Serialize,Deserialize};
//use async_graphql::{Context, Enum, Interface, SimpleObject, Object, Result};
use async_graphql::{SimpleObject};

use crate::utils::load_file;

#[derive(Clone, Debug, Serialize, Deserialize, Default, SimpleObject)]
pub struct LangItems {
  pub data: HashMap<String, String>,
  pub main: HashMap<String, String>,
  pub forms: HashMap<String, String>,
}

impl LangItems {
  pub fn new(root: &str, target: &str, frmt: &str) -> Self {
    if target.is_empty() {
      return LangItems::default();
    }
    let lang_path=format!("{}/{}.{}",&root,&target,&frmt);
    let content = load_file(&lang_path).unwrap_or_else(|e|{
       eprintln!("{}",e);
       String::from("")
    });
    let lang: LangItems = match frmt {
      "yaml" => serde_yaml::from_str(&content)
        .unwrap_or_else(|e|{
          eprintln!("{} Failed parse {} content from {}",e,&frmt,&lang_path);
          LangItems::default()
        }),
      "json" => serde_json::from_str(&content)
        .unwrap_or_else(|e|{
          eprintln!("{} Failed parse {} content from {}",e,&frmt,&lang_path);
          LangItems::default()
        }),
      "toml" => toml::from_str(&content)
        .unwrap_or_else(|e|{
          eprintln!("{} Failed parse {} content from {}",e,&frmt,&lang_path);
          LangItems::default()
        }),
      _ => LangItems::default(),
    };
    // dbg!(&lang);
    println!("Lang loaded from {}", &lang_path);
    lang
  }
  pub fn lng_t (&self, ky: &str, ctx: &str) -> String {
    let mut txt = String::from(ky);
    if ky.is_empty() {
      return txt;
    }
    let op_value = match ctx {
      "main" => self.main.get(ky),
      "data" => self.data.get(ky),
      "forms" => self.forms.get(ky),
      _ => return txt,
    };
    if let Some(item) = op_value {
      txt = String::from(item);
    } 
    txt
  }
  pub fn get_items_section(&self, section: &str) -> HashMap<String, String> {
    match section {
      "main" => self.main.to_owned(),
      "data" => self.data.to_owned(),
      "forms" => self.forms.to_owned(),
      _ => HashMap::new(),
    }
  }
  pub fn get_items_str(&self, section: &str) -> String {
    match section {
      "main" => serde_json::to_string(&self.main),
      "data" => serde_json::to_string(&self.data),
      "forms" => serde_json::to_string(&self.forms),
      _ => serde_json::to_string(&self),
    }.unwrap_or_else(|_|String::from(""))
  }
}

#[cfg(test)]
mod tests {
  use super::*;
  use anyhow::{Result};

  pub fn load_lang_content() -> LangItems {
    let data_path = &envmnt::get_or("DATA_LANG_PATH", ".");
    let data_target = &envmnt::get_or("DATA_LANG_TARGET", "es");
    let data_frmt = &envmnt::get_or("DATA_LANG_FORMAT", "yaml");
    let data_section = &envmnt::get_or("DATA_LANG_SECTION", "");
    println!("Data path: {}/{} in {} format.", &data_path, &data_target, &data_frmt);
    println!("Section {} ", &data_section);
    let lang_items = LangItems::new(&data_path,&data_target,&data_frmt);
    if data_section.is_empty() {
      println!("Loaded items: main ({}) data ({}) forms ({})",
        &lang_items.main.len(), &lang_items.data.len(), &lang_items.forms.len());
    } else {
      println!("Loaded {} items", &lang_items.get_items_section(&data_section).len());
    }
    lang_items
  }

  #[test]
  /// Use environment vars:  DATA_LANG_PATH DATA_LANG_TARGET DATA_LANG_FORMAT DATA_LANG_OUT_FORMAT DATA_LANG_SECTION
  /// lang_test.sh Kloud/ta/langs es.yaml yaml toml main 
  fn load_lang() -> Result<()> {
    let lang_items: LangItems = load_lang_content();
    assert_ne!(lang_items.main.len(), 0);
    assert_ne!(lang_items.data.len(), 0);
    assert_ne!(lang_items.forms.len(), 0);

    let out_format = &envmnt::get_or("DATA_LANG_OUT_FORMAT", "");
    let data_section = &envmnt::get_or("DATA_LANG_SECTION", "");
    if ! out_format.is_empty() {
      println!("Out to {}: ", &out_format);
      let str_data = lang_items.get_items_str(&data_section);
      println!("{}",&str_data);
      assert_ne!(str_data.is_empty(),true)
    }
    Ok(())
  }
}